/*
 *  Bezier.c
 *  esame-06-04-04
 *
 *  Created by Simone Portuesi on 02/04/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */
 
 /* Universita Roma 3, DIA, Informatica Grafica */

/* Curve e superfici di bezier */


/* Include GL, GLU & GLUT */
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#endif

#ifdef linux
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

/* Include some standard stuff */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "Bezier.h"

/* Disegna curva di bezier, di ordine N_CtrlPnts (grado  N_CtrlPnts-1), 
   punti di controllo contenuti in area di memoria puntata da V_CtrlPnts
   approssimandola con una polilinea di steps segmenti
   
   V_CtrlPnts - puntatore ad un vettore di punti in coordinate OMOGENEE (4 double per punto) descriventi i punti di controllo 
   N_CtrlPnts - Ordine della curva di bezier (grado  N_CtrlPnts-1), quindi V_CtrlPnts deve contenere 4*N_CtrlPnts double
   step - numero di segmenti nella polilinea approssimazione */
void BezierCurve(GLdouble *V_CtrlPnts, int N_CtrlPnts, int steps) {

	glMap1d(GL_MAP1_VERTEX_4, 0, 1, 4,  N_CtrlPnts, V_CtrlPnts); 
	glEnable(GL_MAP1_VERTEX_4); 
	
	glMapGrid1d(steps, 0, 1);
	
	glEvalMesh1(GL_LINE, 0, steps);
	
	glDisable(GL_MAP1_VERTEX_4); 
	
}

/* Disegna superfice di bezier di ordine N_CtrlPnts in direzione u e M_CtrlPnts in direzione v, aprossimando con griglia step*step
   V_CtrlPnts - puntatore ad un matrice di punti in coordinate OMOGENEE (4 double per punto) descriventi i punti di controllo
                p-0-0 --- p-0-1 --- ... p_0-N_CtrlPnts
				|         |             |
				p-1-0     p-1-1 --- ... p_1-N_CtrlPnts
				|         |             |
				...       ...           ...
				|         |             |
                p-M_CtrlPnts-0 ....     p-M_CtrlPnts-N_CtrlPnts
				
				{ { { p-0-0-x, p-0-0-y, p-0-0-z, p-0-0-w }, ... , 
					{ p_0-N_CtrlPnts-x, p_0-N_CtrlPnts-y, p_0-N_CtrlPnts-z, p_0-N_CtrlPnts-w } },
				  ...
				  { { p-M_CtrlPnts-0-x, p-M_CtrlPnts-0-y, p-M_CtrlPnts-0-z, p-M_CtrlPnts-0-w }, ... ,
				    { p-M_CtrlPnts-N_CtrlPnts-x, p-M_CtrlPnts-N_CtrlPnts-y, p-M_CtrlPnts-N_CtrlPnts-z, p-M_CtrlPnts-N_CtrlPnts-w } } }
				
			      
   N_CtrlPnts, M_CtrlPnts - Ordine della bezier in direzione u e v, quindi V_CtrlPnts deve contenere N_CtrlPnts*M_CtrlPnts*4 double
   n - numero di segmenti nella griglia di aprossimazione */
void BezierSurface(GLdouble *V_CtrlPnts, int N_CtrlPnts, int M_CtrlPnts, int steps) {

   glMap2d(GL_MAP2_VERTEX_4, 
			0, 1, 4, N_CtrlPnts ,  
			0, 1, 4* N_CtrlPnts ,  M_CtrlPnts,
			V_CtrlPnts);

   glEnable(GL_MAP2_VERTEX_4); 
   glEnable(GL_AUTO_NORMAL); 
   
   glMapGrid2f(steps, 0.0, 1.0, steps, 0.0, 1.0); 

   glEvalMesh2(GL_FILL, 0, steps, 0, steps); 

   glDisable(GL_MAP2_VERTEX_4); 
   glDisable(GL_AUTO_NORMAL); 


}



	
