/*
 *  Bezier.h
 *  esame-06-04-04
 *
 *  Created by Simone Portuesi on 02/04/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

/* Universita Roma 3, DIA, Informatica Grafica */

/* Curve e superfici di bezier */

/* Disegna curva di bezier, di ordine N_CtrlPnts (grado  N_CtrlPnts-1), 
   punti di controllo contenuti in area di memoria puntata da V_CtrlPnts
   approssimandola con una polilinea di steps segmenti
   
   V_CtrlPnts - puntatore ad un vettore di punti in coordinate OMOGENEE (4 double per punto) descriventi i punti di controllo 
   N_CtrlPnts - Ordine della curva di bezier (grado  N_CtrlPnts-1), quindi V_CtrlPnts deve contenere 4*N_CtrlPnts double
   step - numero di segmenti nella polilinea approssimazione */
void BezierCurve(GLdouble *V_CtrlPnts, int N_CtrlPnts, int steps);

/* Disegna superfice di bezier di ordine N_CtrlPnts in direzione u e M_CtrlPnts in direzione v, aprossimando con griglia step*step
   V_CtrlPnts - puntatore ad un matrice di punti in coordinate OMOGENEE (4 double per punto) descriventi i punti di controllo
                p-0-0 --- p-0-1 --- ... p_0-N_CtrlPnts
				|         |             |
				p-1-0     p-1-1 --- ... p_1-N_CtrlPnts
				|         |             |
				...       ...           ...
				|         |             |
                p-M_CtrlPnts-0 ....     p-M_CtrlPnts-N_CtrlPnts
				
				{ { { p-0-0-x, p-0-0-y, p-0-0-z, p-0-0-w }, ... , 
					{ p_0-N_CtrlPnts-x, p_0-N_CtrlPnts-y, p_0-N_CtrlPnts-z, p_0-N_CtrlPnts-w } },
				  ...
				  { { p-M_CtrlPnts-0-x, p-M_CtrlPnts-0-y, p-M_CtrlPnts-0-z, p-M_CtrlPnts-0-w }, ... ,
				    { p-M_CtrlPnts-N_CtrlPnts-x, p-M_CtrlPnts-N_CtrlPnts-y, p-M_CtrlPnts-N_CtrlPnts-z, p-M_CtrlPnts-N_CtrlPnts-w } } }
				
			      
   N_CtrlPnts, M_CtrlPnts - Ordine della bezier in direzione u e v, quindi V_CtrlPnts deve contenere N_CtrlPnts*M_CtrlPnts*4 double
   n - numero di segmenti nella griglia di aprossimazione */
void BezierSurface(GLdouble *V_CtrlPnts, int N_CtrlPnts, int M_CtrlPnts, int steps);

