/*
 *  ProfileRotate.c
 *  esame-06-04-04
 *
 *  Created by Simone Portuesi on 08/03/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

/* Universita Roma 3, DIA, Informatica Grafica Esercitazione */

/* Rotazione profilo                                   */

/* Include GL, GLU & GLUT */
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#endif

#ifdef linux
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

/* Include some standard stuff */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Include my files */
#include "utility.h"
int shownormals=0;

/* Ruota la polilinea, descritta da profile ed n_profile, sull'asse z, in n passi d'approssimazione
   calcola normali secondo normale calcolata da polilinea e rotazione
   mappa le coordinate di texture su (u,v)=[(0,0),(1,1)]

   profile - puntatore ad un vettore di coppie di float, ogni coppia descrive un punto della polilinea 
   n_profile - numero di punti sulla polilinea, quindi profile deve puntare a zona di memoria contenente 2*nprofile float
   n - numero di punti del poligono approssimante
*/
void rotateprofile(GLfloat profile[][2], int n_profile, int n) {
	int i,j,k;
	GLfloat angle0,angle1,angle1r;
	GLfloat step=2.0 * M_PI / n;
	GLfloat profileN[2];
	GLfloat d;
		
	/* For each segment */
	for(i=0;i<n_profile-1;i++) {
		/* For each slice */
		for(j=0;j<n;j++) {
			/* Calculate angles */
			angle0=j * step;
			angle1=((j+1) % n) * step;
			angle1r=(j+1) * step;
			
			/* Calculate normal of profile */
			profileN[0] = profile[i][1] -  profile[i+1][1];
			profileN[1] = profile[i+1][0] -  profile[i][0];
			d=sqrt(profileN[0] * profileN[0] + profileN[1] * profileN[1]);
			profileN[0] /=d;
			profileN[1] /=d;
			
			/* Calculate profile distance from 0 */
			if(shownormals) {
				glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
				glVertex3f(profile[i][0] * cos(angle0),
						profile[i][1] ,
						profile[i][0] * sin(angle0));
				glVertex3f((profile[i][0] + profileN[0])  * cos(angle0),
						profile[i][1] + profileN[1],
						(profile[i][0] + profileN[0])  * sin(angle0) );
			
				glEnd();
				glBegin(GL_LINES);
				glVertex3f(profile[i+1][0] * cos(angle1),
						profile[i+1][1] ,
						profile[i+1][0] * sin(angle1));
				glVertex3f((profile[i+1][0] + profileN[0])  * cos(angle1),
						profile[i+1][1] + profileN[1],
						(profile[i+1][0] + profileN[0])  * sin(angle1) );
			
				glEnd();
				glEnable(GL_LIGHTING);
			}
			
			
			/* Draw face, updating the angle */
			glBegin(GL_POLYGON);
			glPolygonMode(GL_FRONT,GL_FILL);
			
			/* first point of profile projected on first angle */
			glTexCoord2f(angle0 / (2 * M_PI),
						 1 - ((GLfloat) i)/((GLfloat) n_profile));
			glNormal3f(profileN[0] * cos(angle0),
					   profileN[1],
					   profileN[0] * sin(angle0));
			glVertex3f( profile[i][0] * cos(angle0),
			            profile[i][1],
						profile[i][0] * sin(angle0) );
						
			/* second point of profile projected on first angle */
			glTexCoord2f(angle0 / (2 * M_PI),
						 1 - ((GLfloat) (i+1))/((GLfloat) n_profile));
			glNormal3f(profileN[0] * cos(angle0),
					   profileN[1],
					   profileN[0] * sin(angle0));
			glVertex3f( profile[i+1][0] * cos(angle0),
			            profile[i+1][1],
						profile[i+1][0] * sin(angle0) );
									
			/* second point of profile projected on second angle */		
			glTexCoord2f(angle1r / (2 * M_PI),
						 1 - ((GLfloat) (i+1))/((GLfloat) n_profile));				
			glNormal3f(profileN[0] * cos(angle1),
					   profileN[1],
					   profileN[0] * sin(angle1));
			glVertex3f( profile[i+1][0] * cos(angle1),
			            profile[i+1][1],
						profile[i+1][0] * sin(angle1));
			
			/* first point of profile projected on second angle */
			glTexCoord2f(angle1r / (2 * M_PI),
						 1 - ((GLfloat) i)/((GLfloat) n_profile));	
			glNormal3f(profileN[0] *  cos(angle1),
					   profileN[1],
					   profileN[0]*  sin(angle1));
			glVertex3f( profile[i][0] * cos(angle1),
			            profile[i][1],
						profile[i][0] * sin(angle1));
			
			glEnd();
			
			
		}
	}
} 


